{% load timepiece_tags %}
{% block quick-clock-in %}
    {% if perms.entries.can_clock_in %}
        <li class="dropdown">
            <a href="#" class="dropdown-toggle" data-toggle="dropdown"><i class="icon-time icon-white"> </i><b class="caret"></b> </a>
            <ul class="dropdown-menu">
                <li><a href="{% url 'clock_in' %}">Clock in...</a></li>
                <li><a href="{% url 'create_entry' %}">Add Entry...</a></li>
                {% if work_projects %}<li class="divider"></li>{% endif %}
                {% for project in work_projects %}
                    <li><a href="{% url 'clock_in' %}?project={{ project.id }}">{{ project }}</a></li>
                {% endfor %}
                {% if leave_projects %}<li class="divider"></li>{% endif %}
                {% for project in leave_projects %}
                    <li><a href="{% url 'clock_in' %}?project={{ project.id }}">{{ project }}</a></li>
                {% endfor %}
            </ul>
        </li>
    {% endif %}
{% endblock quick-clock-in %}
